#ifndef __GPIO_H__
#define __GPIO_H__

#define NUM_GPIOS	16

#define MEM_ERR		(-1)
#define FILE_ERR	(-2)
#define SIGN_ERR	(-3)
#define INPUT_ERR	(-4)

#define GPIO_KILL_PIN	4

#define MAX_SYSFS_NAME_LEN	120
#define MAX_COMMAND_LEN		10

#define DIRECTION_IN		0
#define DIRECTION_OUT		1
#define LEVEL_LOW		0
#define LEVEL_HIGH		1

static int never_export[] = {GPIO_KILL_PIN};

typedef struct circuit {
	int pins[NUM_GPIOS]; 	/* set of connected pins */
	int npins;		/* amount of pins in set */
	int never;		/* never play with this pin */
	int avail;		/* pin exported/unexported */
	int active_low;		/* "0" - signal is "0", "1" - signal is "1" */
} circuit_t;

typedef struct jumper {
	int pin;	/* second pin in jumper */
	int live;	/* connected */
	int never;	/* never play with this pin */
	int avail;	/* pin exported/unexported */
	int active_low; /* "0" - signal is "0", "1" - signal is "1" */
} jumper_t;

extern int gpio_pin_open(int pin);
extern int gpio_pin_set_direction(int pin, int dir);
extern int gpio_pin_set_level(int pin, int level);
extern int gpio_pin_get_value(int pin);
extern int gpio_pin_get_active_low(int pin);
extern void gpio_pin_close(int pin);

extern int pin_is_valid(int pin);

#endif /* __GPIO_H__ */
