#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <signal.h>

#include "gpio.h"

#define SYSFS_PATH "/sys/class/gpio"

int gpio_pin_open(int pin) {

	char sysfs[MAX_SYSFS_NAME_LEN];
	char export_name[MAX_SYSFS_NAME_LEN];
	char command[MAX_COMMAND_LEN];

	FILE *export = NULL;

	sprintf(sysfs, SYSFS_PATH);
	sprintf(export_name, "%s/export", sysfs);
	sprintf(command, "%d", pin);

	/* export pin */
	export = fopen(export_name, "w");
	if (!export)
		return FILE_ERR;
	
	fputs(command, export);
	fclose(export);

	return 0;
} /* gpio_pin_open */

int gpio_pin_set_direction(int pin, int dir) {

	char sysfs[MAX_SYSFS_NAME_LEN];
	char direction_name[MAX_SYSFS_NAME_LEN];
	char command[MAX_COMMAND_LEN];

	FILE *direction = NULL;

	sprintf(sysfs, SYSFS_PATH);
	sprintf(direction_name, "%s/GPIO%d/direction", sysfs, pin);

	direction = fopen(direction_name, "w");
	if (!direction) {
	  	printf("failed to open file\n");
		return FILE_ERR;
	}

	if (dir == DIRECTION_OUT) {
		sprintf(command, "out");
	} else if (dir == DIRECTION_IN) {
		sprintf(command, "in");
	} else {
		fclose(direction);
		return INPUT_ERR;
	} /* if */

	fputs(command, direction);
	fclose(direction);

	return 0;
} /* gpio_pin_set_direction */

int gpio_pin_set_level(int pin, int level) {

	char sysfs[MAX_SYSFS_NAME_LEN];
	char value_name[MAX_SYSFS_NAME_LEN];
	char command[MAX_COMMAND_LEN];

	FILE *value = NULL;

	sprintf(sysfs, SYSFS_PATH);
	sprintf(value_name, "%s/GPIO%d/value", sysfs, pin);

	value = fopen(value_name, "w");
	if (!value)
		return FILE_ERR;

	if (level == LEVEL_LOW) {
		sprintf(command, "0");
	} else if (level == LEVEL_HIGH) {
		sprintf(command, "1");
	} else {
		fclose(value);
		return INPUT_ERR;
	} /* if */

	fputs(command, value);
	fclose(value);

	return 0;
} /* gpio_pin_set_level */

int gpio_pin_get_value(int pin) {

	char sysfs[MAX_SYSFS_NAME_LEN];
	char value_name[MAX_SYSFS_NAME_LEN];
	char str[MAX_COMMAND_LEN];

	FILE *value = NULL;

	sprintf(sysfs, SYSFS_PATH);
	sprintf(value_name, "%s/GPIO%d/value", sysfs, pin);

	value = fopen(value_name, "r");
	if (!value)
		return FILE_ERR;

	fgets(str, 2, value);
	fclose(value);

	return atoi(str);
} /* gpio_pin_get_value */

int gpio_pin_get_active_low(int pin) {

	char sysfs[MAX_SYSFS_NAME_LEN];
	char active_low_name[MAX_SYSFS_NAME_LEN];
	char str[MAX_COMMAND_LEN];

	FILE *active_low = NULL;

	sprintf(sysfs, SYSFS_PATH);
	sprintf(active_low_name, "%s/GPIO%d/active_low", sysfs, pin);

	active_low = fopen(active_low_name, "r");
	if (!active_low)
		return FILE_ERR;

	fgets(str, 2, active_low);
	fclose(active_low);

	return atoi(str);
} /* gpio_pin_get_active_low */

void gpio_pin_close(int pin) {

	char sysfs[MAX_SYSFS_NAME_LEN];
	char unexport_name[MAX_SYSFS_NAME_LEN];
	char command[MAX_COMMAND_LEN];

	FILE *unexport = NULL;

	sprintf(sysfs, SYSFS_PATH);
	sprintf(unexport_name, "%s/unexport", sysfs);
	sprintf(command, "%d", pin);

	/* export pin */
	unexport = fopen(unexport_name, "w");
	if (!unexport)
		return;

	fputs(command, unexport);
	fclose(unexport);

	return;
} /* gpio_pin_close */

int pin_is_valid(int pin) {

	int i;

	for (i = 0; i < sizeof(never_export) / sizeof(int); i++) {
		if (never_export[i] == pin) {
			return 0;
		} /* if */
	} /* for */

	return 1;
} /* pin_is_valid */
