#!/bin/bash

umask 002

if [[ -z $Path_SICw ]]; then
  if ( [ -f /proc/sicread ] && [ -f /proc/sicregs ] ); then
    Path_SICw=/proc/sicread
    Path_SICr=/proc/sicregs
  elif  ( [ -f /proc/sicread ] && [ -f /proc/sicwrite ] ); then
    Path_SICw=/proc/sicread
    Path_SICr=/proc/sicread
  fi
  [[ ! -f $Path_SICw ]] && exit 1
fi

NODES_NUM=$(ls /sys/devices/system/node/ | grep node | wc -l)

#default schema for FOUR nodes
CPU_LIST="0 1 2 3"
LNK_LIST="1 2 3"

if [[ $NODES_NUM -eq 2 ]]; then
    CPU_LIST="0 1"
    LNK_LIST="1"
fi

if [[ $NODES_NUM -eq 1 ]]; then
    echo "Only single CPU found - test SKIPPED"
    exit 2
fi


READ_ATTEMPT_NUMBER=20
get_reg() {
    local r
    local s
    local a
    local N=0
    local hex1=$(printf "0x%x" $1)
    while true; do
	((N++))
	echo ${hex1} > $Path_SICw
	s=`cat ${Path_SICr} | grep "node: $2 reg"`
	a=(`echo ${s,,} | awk '{print $4}' | tr '():' ' '`)
	if [[ $hex1 = ${a[0]} ]]; then
	    break
	fi
	if [[ -z ${a[0]} ]]; then
	    break
	fi
	if ((N>${READ_ATTEMPT_NUMBER})); then
	    break
	fi
	sleep .1
    done
    echo ${s}
}

ipl=("error" "IPL_A" "IPL_B" "IPL_C")

# Checking
#echo "0x0604" > $Path_SICw
#status_links[0]=`cat $Path_SICr | grep "node: 0 reg"`
#status_links[1]=`cat $Path_SICr | grep "node: 1 reg"`
#status_links[2]=`cat $Path_SICr | grep "node: 2 reg"`
#status_links[3]=`cat $Path_SICr | grep "node: 3 reg"`
status_links[0]=`get_reg "0x604" 0`
status_links[1]=`get_reg "0x604" 1`
status_links[2]=`get_reg "0x604" 2`
status_links[3]=`get_reg "0x604" 3`

#echo "0x0644" > $Path_SICw
#status_links[4]=`cat $Path_SICr | grep "node: 0 reg"`
#status_links[5]=`cat $Path_SICr | grep "node: 1 reg"`
#status_links[6]=`cat $Path_SICr | grep "node: 2 reg"`
#status_links[7]=`cat $Path_SICr | grep "node: 3 reg"`
status_links[4]=`get_reg "0x644" 0`
status_links[5]=`get_reg "0x644" 1`
status_links[6]=`get_reg "0x644" 2`
status_links[7]=`get_reg "0x644" 3`

#echo "0x0684" > $Path_SICw
#status_links[8]=`cat $Path_SICr | grep "node: 0 reg"`
#status_links[9]=`cat $Path_SICr | grep "node: 1 reg"`
#status_links[10]=`cat $Path_SICr | grep "node: 2 reg"`
#status_links[11]=`cat $Path_SICr | grep "node: 3 reg"`
status_links[8]=`get_reg "0x684" 0`
status_links[9]=`get_reg "0x684" 1`
status_links[10]=`get_reg "0x684" 2`
status_links[11]=`get_reg "0x684" 3`

err=0
for cpu in ${CPU_LIST}; do
	for lnk in ${LNK_LIST}; do
		warn=0
		let "i = 4 * (${lnk} - 1) + ${cpu}"
		if [ -z "${status_links[${i}]}" ]; then
			echo "Error! Cannot read status for ${ipl[${lnk}]} on NODE_${cpu}";
			err=1
			continue;
		fi
		status_links_csr=`echo "${status_links[${i}]}" | awk '{print $5}'`
		csr_cmd_act=$(( (${status_links_csr} & 0x00000080) >> 7 ))
		csr_link_state=$(( (${status_links_csr} & 0x80000000) >> 31 ))
		csr_ltssm_state=`printf "0x%x" $(( (${status_links_csr} & 0x00070000) >> 16 ))`
		csr_link_scale=`printf "0x%x" $(( (${status_links_csr} & 0x0000000F) >> 0 ))`
		csr_link_width=`printf "0x%x" $(( (${status_links_csr} & 0x0F000000) >> 24 ))`

		if [ "${csr_link_scale}" != "0xf" ]; then
			echo "Error! Wrong status for ${ipl[${lnk}]} on NODE_${cpu}: not full scale=${csr_link_scale}/0xf (CSR=${status_links_csr})";
			err=1
			continue;
		fi

		if [ "${csr_cmd_act}" == "1" ]; then
			echo "Error! Wrong status for ${ipl[${lnk}]} on NODE_${cpu}: command executing (CSR=${status_links_csr})";
			err=1
			continue;
		fi
		if [ "${csr_link_state}" != "1" ]; then
			echo "Warning! Wrong status for ${ipl[${lnk}]} on NODE_${cpu}: link is not active (CSR=${status_links_csr})";
			warn=1
			err=1
		fi
		if [ "${csr_ltssm_state}" != "0x3" ]; then
			echo "Error! Wrong status for ${ipl[${lnk}]} on NODE_${cpu}: ltssm is not linkup (CSR=${status_links_csr})";
			warn=1
			err=1
		fi
		if [ "${csr_link_width}" != "0xf" ]; then
			echo "Error! Wrong status for ${ipl[${lnk}]} on NODE_${cpu}: not full width=${csr_link_width}/0xf (CSR=${status_links_csr})";
			warn=1
			err=1
			wdth=`echo ${csr_link_width} | sed s/0x//`
			cl=""
			[[ "$(( (${wdth} & 0x1) >> 0 ))" == "1" ]] || cl="$cl 0 1 2 3"
			[[ "$(( (${wdth} & 0x2) >> 1 ))" == "1" ]] || cl="$cl 4 5 6 7"
			[[ "$(( (${wdth} & 0x4) >> 2 ))" == "1" ]] || cl="$cl 8 9 10 11"
			[[ "$(( (${wdth} & 0x8) >> 3 ))" == "1" ]] || cl="$cl 12 13 14 15"
			let "pair_cpu = (${cpu} + ${lnk}) % 4"
			if [ "${lnk}" == "1" ]; then
				pair_lnk="3"
			elif [ "${lnk}" == "3" ]; then
				pair_lnk="1"
			else
				pair_lnk="${lnk}"
			fi
			echo "       advice: try to check lanes${cl} on link CPU_${cpu}_${ipl[${lnk}]} <-> CPU_${pair_cpu}_${ipl[${pair_lnk}]}";
		fi

		[[ "${warn}" != "1" ]] && echo "OK! Good status for ${ipl[${lnk}]} on NODE_${cpu} (CSR=${status_links_csr})";
	done
done

exit $err
