#!/bin/bash

umask 002

[[ ! -z $RUN_TRACE ]] && { echo "$0 RUN_TRACE ~~~~~~~~~~~~~~~~~~~~"; set -x; }

if [[ -z $Path_SICw ]]; then
  if ( [ -f /proc/sicread ] && [ -f /proc/sicregs ] ); then
    Path_SICw=/proc/sicread
    Path_SICr=/proc/sicregs
  elif  ( [ -f /proc/sicread ] && [ -f /proc/sicwrite ] ); then
    Path_SICw=/proc/sicread
    Path_SICr=/proc/sicread
  fi
  [[ ! -f $Path_SICw ]] && exit 2
fi

NODES_NUM=$(ls /sys/devices/system/node/ | grep node | wc -l)


#default schema for FOUR nodes
CPU_LIST="0 1 2 3"
LNK_LIST="1 2 3"

if [[ $NODES_NUM -eq 2 ]]; then
    CPU_LIST="0 1"
    LNK_LIST="1"
fi

if [[ $NODES_NUM -eq 1 ]]; then
    echo "Only single CPU found - test SKIPPED"
    exit 2
fi

# Routing settings (depends on motherboard)
ipl_pairs=("error" "1" "3" "2")		# ipl_pairs[ipl_N] shows number of IPL that connected to IPL_N
cpu0_pairs=("error" "1" "2" "3")	# cpu0_pairs[ipl_N] shows number of NODE that connected to NODE_0 through IPL_N
cpu1_pairs=("error" "0" "3" "2")	# cpu1_pairs[ipl_N] shows number of NODE that connected to NODE_1 through IPL_N
cpu2_pairs=("error" "3" "1" "0")	# cpu2_pairs[ipl_N] shows number of NODE that connected to NODE_2 through IPL_N
cpu3_pairs=("error" "2" "0" "1")	# cpu3_pairs[ipl_N] shows number of NODE that connected to NODE_3 through IPL_N
reverse_ipla_cpus=("0" "1" "0" "1")	# reverse_ipla_cpus[cpu_N] shows if IPL_A data lanes are reverseed on NODE_N (depends on motherboard)

READ_ATTEMPT_NUMBER=20
get_reg() {
    local r
    local s
    local a
    local N=0
    local hex1=$(printf "0x%x" $1)
    while true; do
	((N++))
	echo ${hex1} > $Path_SICw
	s=`cat ${Path_SICr} | grep "node: $2 reg"`
	a=(`echo ${s,,} | awk '{print $4}' | tr '():' ' '`)
	if [[ $hex1 = ${a[0]} ]]; then
	    break
	fi
	if [[ -z ${a[0]} ]]; then
	    break
	fi
	if ((N>${READ_ATTEMPT_NUMBER})); then
	    break
	fi
	sleep .1
    done
    echo ${s}
}

# Checking
#echo "0x0604" > $Path_SICw
#status_links[0]=`cat $Path_SICr | grep "node: 0 reg"`
#status_links[1]=`cat $Path_SICr | grep "node: 1 reg"`
#status_links[2]=`cat $Path_SICr | grep "node: 2 reg"`
#status_links[3]=`cat $Path_SICr | grep "node: 3 reg"`
status_links[0]=`get_reg "0x604" 0`
status_links[1]=`get_reg "0x604" 1`
status_links[2]=`get_reg "0x604" 2`
status_links[3]=`get_reg "0x604" 3`

#echo "0x0644" > $Path_SICw
#status_links[4]=`cat $Path_SICr | grep "node: 0 reg"`
#status_links[5]=`cat $Path_SICr | grep "node: 1 reg"`
#status_links[6]=`cat $Path_SICr | grep "node: 2 reg"`
#status_links[7]=`cat $Path_SICr | grep "node: 3 reg"`
status_links[4]=`get_reg "0x644" 0`
status_links[5]=`get_reg "0x644" 1`
status_links[6]=`get_reg "0x644" 2`
status_links[7]=`get_reg "0x644" 3`

#echo "0x0684" > $Path_SICw
#status_links[8]=`cat $Path_SICr | grep "node: 0 reg"`
#status_links[9]=`cat $Path_SICr | grep "node: 1 reg"`
#status_links[10]=`cat $Path_SICr | grep "node: 2 reg"`
#status_links[11]=`cat $Path_SICr | grep "node: 3 reg"`
status_links[8]=`get_reg "0x684" 0`
status_links[9]=`get_reg "0x684" 1`
status_links[10]=`get_reg "0x684" 2`
status_links[11]=`get_reg "0x684" 3`

ipl_name=("error" "IPL_A" "IPL_B" "IPL_C")

err=0
for cpu in ${CPU_LIST}; do
	for lnk in ${LNK_LIST}; do
		warn=0
		let "i = 4 * (${lnk} - 1) + ${cpu}"
		if [ -z "${status_links[${i}]}" ]; then
			echo "Error! Cannot read status for ${ipl_name[${lnk}]} on NODE_${cpu}";
			err=1
			continue;
		fi
		status_links_csr=`echo "${status_links[${i}]}" | awk '{print $5}'`
		csr_cmd_act=$(( (${status_links_csr} & 0x00000080) >> 7 ))
		csr_link_state=$(( (${status_links_csr} & 0x80000000) >> 31 ))
		csr_ltssm_state=`printf "0x%x" $(( (${status_links_csr} & 0x00070000) >> 16 ))`
		csr_link_scale=`printf "0x%x" $(( (${status_links_csr} & 0x0000000F) >> 0 ))`
		csr_link_width=`printf "0x%x" $(( (${status_links_csr} & 0x0F000000) >> 24 ))`

		if [ "${csr_link_scale}" != "0xf" ]; then
			echo "Error! Wrong status for ${ipl_name[${lnk}]} on NODE_${cpu}: not full scale=${csr_link_scale}/0xf (CSR=${status_links_csr})";
			err=1
			continue;
		fi

		if [ "${csr_cmd_act}" == "1" ]; then
			echo "Error! Wrong status for ${ipl_name[${lnk}]} on NODE_${cpu}: command executing (CSR=${status_links_csr})";
			err=1
			continue;
		fi
		if [ "${csr_link_state}" != "1" ]; then
			echo "Warning! Wrong status for ${ipl_name[${lnk}]} on NODE_${cpu}: link is not active (CSR=${status_links_csr})";
			warn=1
			err=1
		fi
		if [ "${csr_ltssm_state}" != "0x3" ]; then
			echo "Warning! Wrong status for ${ipl_name[${lnk}]} on NODE_${cpu}: ltssm is not linkup (CSR=${status_links_csr})";
			warn=1
			err=1
		fi
		if [ "${csr_link_width}" != "0xf" ]; then
			echo "Warning! Wrong status for ${ipl_name[${lnk}]} on NODE_${cpu}: not full width=${csr_link_width}/0xf (CSR=${status_links_csr})";
			warn=1
			err=1
			wdth=`echo ${csr_link_width} | sed s/0x//`

			cl_true=""
			[[ "$(( (${wdth} & 0x1) >> 0 ))" == "1" ]] || cl_true="$cl_true 0 1 2 3"
			[[ "$(( (${wdth} & 0x2) >> 1 ))" == "1" ]] || cl_true="$cl_true 4 5 6 7"
			[[ "$(( (${wdth} & 0x4) >> 2 ))" == "1" ]] || cl_true="$cl_true 8 9 10 11"
			[[ "$(( (${wdth} & 0x8) >> 3 ))" == "1" ]] || cl_true="$cl_true 12 13 14 15"
			cl_rev=""
			[[ "$(( (${wdth} & 0x1) >> 0 ))" == "1" ]] || cl_rev="$cl_rev 15 14 13 12"
			[[ "$(( (${wdth} & 0x2) >> 1 ))" == "1" ]] || cl_rev="$cl_rev 11 10 9 8"
			[[ "$(( (${wdth} & 0x4) >> 2 ))" == "1" ]] || cl_rev="$cl_rev 7 6 5 4"
			[[ "$(( (${wdth} & 0x8) >> 3 ))" == "1" ]] || cl_rev="$cl_rev 3 2 1 0"

			pair_lnk="${ipl_pairs[${lnk}]}"

			if [ "${cpu}" == "0" ]; then
				pair_cpu="${cpu0_pairs[${lnk}]}"
			elif [ "${cpu}" == "1" ]; then
				pair_cpu="${cpu1_pairs[${lnk}]}"
			elif [ "${cpu}" == "2" ]; then
				pair_cpu="${cpu2_pairs[${lnk}]}"
			elif [ "${cpu}" == "3" ]; then
				pair_cpu="${cpu3_pairs[${lnk}]}"
			else
				pair_cpu="error"
			fi

			if [ "${lnk}" == "1" ] && [ "${reverse_ipla_cpus[${cpu}]}" == "1" ]; then
				cl="${cl_rev}"
				if [ "${pair_lnk}" == "1" ] && [ "${reverse_ipla_cpus[${pair_cpu}]}" == "1" ]; then
					note=""
				else
					note="(be carefull, lanes are reversed, so check${cl_true} on another side)"
				fi
			else
				cl="${cl_true}"
				if [ "${pair_lnk}" == "1" ] && [ "${reverse_ipla_cpus[${pair_cpu}]}" == "1" ]; then
					note="(be carefull, lanes are reversed, so check${cl_rev} on another side)"
				else
					note=""
				fi
			fi

			echo "       advice: try to check lanes${cl} on link CPU_${cpu}_${ipl_name[${lnk}]} <-> CPU_${pair_cpu}_${ipl_name[${pair_lnk}]} ${note}";
		fi

		[[ "${warn}" != "1" ]] && echo "OK! Good status for ${ipl_name[${lnk}]} on NODE_${cpu} (CSR=${status_links_csr})";
	done
done

[[ "${err}" == "1" ]] && exit 1
exit 0
