#!/bin/bash

umask 002

MEM_CHANNELS=4
SUPPORT_TYPE="E16C|E12C|E8C2|E8C|E2S"
CHECK_MEMORY_PRESENT="0x0"

NO_SEC_REGS=1
if ( [ -f /proc/sicread ] && [ -f /proc/sicregs ] ); then
	Path_SICw=/proc/sicread
	Path_SICr=/proc/sicregs
	Path_SICW=/proc/sicwrite
	NO_SEC_REGS=0
elif  ( [ -f /proc/sicread ] && [ -f /proc/sicwrite ] ); then
	Path_SICw=/proc/sicread
	Path_SICr=/proc/sicread
	Path_SICW=/proc/sicwrite
	NO_SEC_REGS=0
fi

cpu_type=$(cat /proc/cpuinfo | grep "model name" | sort -u | awk '{print $4}')
echo $cpu_type | grep -w -E ${SUPPORT_TYPE} >/dev/null

REG_STR="0x60c 0x64c 0x68c"
if [[ $cpu_type == "E12C" ]]; then
	REG_STR="0x60c"
fi

if (($NO_SEC_REGS)); then
	uname -a
	echo -e "\nError: Not found kernel interface for SIC-registers !\n"
	exit 1
fi

#choose number of memory channels
case "$cpu_type" in
    "E16C" ) MEM_CHANNELS=8;;
    "E12C" ) MEM_CHANNELS=8;;
    "E8C2" ) MEM_CHANNELS=4;;
    "E8C"  ) MEM_CHANNELS=4;;
    "E2S"  ) MEM_CHANNELS=3;;
#    "E1C+") MEM_CHANNELS=2;;
esac

FRES=
FSTR=
READ_ATTEMPT_NUMBER=20

get_reg() {
    local r
    local a
    local N=0
    local hex1=$(printf "0x%x" $1)
    while true; do
	((N++))
	if [[ $cpu_type == "E16C" ]] || [[  $cpu_type == "E12C" ]]; then
		echo "$2 ${hex1} 0x400" > $Path_SICW
	fi
	echo ${hex1} > $Path_SICw
	FSTR=`cat ${Path_SICr} | grep "node: $2 reg"`
	a=(`echo ${FSTR,,} | awk '{print $4}' | tr '():' ' '`)
	r=(`echo ${FSTR,,} | awk '{print $5}'`)
	if [[ $hex1 = ${a[0]} ]]; then
	    break
	fi
	if [[ -z ${a[0]} ]]; then
	    break
	fi
	if ((N>${READ_ATTEMPT_NUMBER})); then
	    break
	fi
#	echo "loop $N -- _${hex1}_ _${a[0]}_ __${a[*]}__"
	sleep .1
    done
    FRES=$(echo ${r})
}


check_ecc_regs() {
    err=0
    cnt=0
    echo "== Check ECC status"
    for i in `seq 0 $1`; do
	echo -e "\tChannel $i"
	if [[ $cpu_type == "E16C" ]] || [[  $cpu_type == "E12C" ]]; then
		d="0x440"
	else
		(( d = 0x400 + 0x40 * $i ))
	fi
	for k in 0 1 2 3; do
	    get_reg $d $k
	    out[$k]=$FRES
	    outstr[$k]=$FSTR
	done

	n=`cat $Path_SICr | wc -l`
	for z in `seq 0 $((n - 1))`; do
	    if [[ ${out[${z}]} != "0x1" ]] && [[ ${out[${z}]} != ${CHECK_MEMORY_PRESENT} ]] ; then
		(( i = ((${cnt} + 1)) * 10 + $z ))
		ecnh[${i}]="Channel: ${cnt} ${outstr[${z}]} <-- ECC ERROR"
		echo -e "\t\t${outstr[${z}]} <-- ECC ERROR"
		(( err = $err + 1 ))
	    else
		echo -e "\t\t${outstr[${z}]}"
	    fi
	done

	(( cnt = $cnt + 1 ))
    done

    return $err
}


print_date() {
    d=`date`
    echo "Current date: $d"
    [[ ! -z $1 ]] && echo "Finish  date: `date -d @$1`"
    echo
}
echo "======================= ======================="
print_date
check_ecc_regs $((MEM_CHANNELS - 1))
if (($?)); then
  RET=1
fi

exit $RET
