#!/bin/bash

umask 002

MEMSIZE=100
CHECKTIME=30
WORKTIME=900

PATH_TO_STDP="/opt/mcst/check/stdp"
if [[ ! -d "/opt/mcst/check/stdp" ]]; then
    PATH_TO_STDP="/opt/mcst/stdp"
fi

PATH_TO_MEM="${PATH_TO_STDP}/tests/main/mem"
MEMFILE="${PATH_TO_MEM}/test/mem"
MEMTEST="${MEMFILE} -s $MEMSIZE -c 0 -q"
NOMEMTEST=0

if [[ ! -f ${MEMFILE} ]] && [[ $NOMEMTEST == "0" ]]; then
   echo "ERROR: no mem test !"
    exit 1
fi

DETAILED_FLAG=0
ONE_ERROR=0
COUNT_ERROR=0
NUMACTL_PRESENT="present"

RET=0

# by default we are ignore empty memory slots
CHECK_MEMORY_PRESENT=0

OPTIONS="hm:c:t:C:MEp:D"
while getopts $OPTIONS OPT
do
	case $OPT in
	D ) DETAILED_FLAG=1;;
 	m ) MEMSIZE=${OPTARG};;
	C ) COUNT_ERROR=${OPTARG};;
	M ) NOMEMTEST=1;;
	E ) ONE_ERROR=1;;
	p )
		# if func call with args
		# than we evalute state "0x0"
		# whats meaning -- "empty memory slots present" -- as error
		CHECK_MEMORY_PRESENT=1;;
	c ) CHECKTIME=${OPTARG};;
	t ) WORKTIME=${OPTARG};;
	h ) echo "
 Usage: $0 [-c checktime] [-t worktime] [-m memsize] [-C count of errors]
	-c - set check time. Default: $CHECKTIME seconds
	-t - set work time. Default: $WORKTIME seconds
	-m - set size for mem test. Default: $MEMSIZE MiB
	-C - count of errors allowed for successful testing
        -p - check memory present in each memory place.
	-E - test termination after detect first error
	-M - turn OFF mem test
	-D - turn ON detailed information
";exit 0;;
	esac
done
MEM_CHANNELS=4

SUPPORT_TYPE="E16C|E12C|E8C2|E8C|E2S"
NEW_TYPE="E16C|E12C"

NO_SIC_REGS=1
if ( [ -f /proc/sicread ] && [ -f /proc/sicregs ] ); then
	Path_SICw=/proc/sicread
	Path_SICr=/proc/sicregs
	Path_SICW=/proc/sicwrite
	NO_SIC_REGS=0
elif  ( [ -f /proc/sicread ] && [ -f /proc/sicwrite ] ); then
	Path_SICw=/proc/sicread
	Path_SICr=/proc/sicread
	Path_SICW=/proc/sicwrite
	NO_SIC_REGS=0
fi

cpu_type=$(cat /proc/cpuinfo | grep "model name" | sort -u | awk '{print $4}')
echo $cpu_type | grep -w -E ${SUPPORT_TYPE} >/dev/null
if (($?)); then
	uname -a
	echo -e "\nSKIP: IPLink manipulatuion is not supported !\n";
	exit 2
fi
if (($NO_SIC_REGS)); then
	uname -a
	echo -e "\nError: Not found kernel interface for IPLink !\n"
	exit 1
fi
ONE_MC_CH_REG=1
echo $cpu_type | grep -w -E ${NEW_TYPE} >/dev/null
if (($?)); then
	ONE_MC_CH_REG=
fi
IPCC_CSR=(0x604 0x644 0x684)
IPCC_STR=(0x60c 0x64c 0x68c)
CSR_CMD_CODE="0x0|0x1|0x2"

#choose number of memory channels
case "$cpu_type" in
    "E16C" ) MEM_CHANNELS=8;;
    "E12C" ) MEM_CHANNELS=8; IPCC_STR="0x60c";;
    "E8C2" ) MEM_CHANNELS=4;;
    "E8C"  ) MEM_CHANNELS=4;;
    "E2S"  ) MEM_CHANNELS=3;;
#    "E1C+") MEM_CHANNELS=2;;
esac

active_flag=0
lsmod | grep "hw_check"
if (($?)); then
	modprobe hw_check
	if (($?)); then
		echo "hw_check is not available"
	else
		active_flag=1
	fi
else
	active_flag=1
fi

if [ ! -z $active_flag ]; then
  PATH_TO_HWMON="/sys/class/hwmon/"
  node=`cat $Path_SICr | wc -l`
  Paths=`find /sys/devices/system/cpu/ -name link_info \
	| sed 's/link_info//g' | sort` 
  echo "$Paths" > paths_file
  for cur_node in `seq 0 $((node - 1))`; do
    String_num=$((1 + $cur_node))
    Path_dir[$cur_node]=`head -n ${String_num} paths_file \
		| tail -n 1 | tr -d "\n"`
    Path_to_link_info[$cur_node]=${Path_dir[$cur_node]}link_info
    Path_to_mem_info[$cur_node]=${Path_dir[$cur_node]}mem_info
    Path_to_mem_rate[$cur_node]=${Path_dir[$cur_node]}mem_rate
  done
fi
#get NUMA numbers
CORENUM=
NUMANUM=
if [ -d /sys/devices/system/node/node0 ]; then
  if [ ! -z `command -v lscpu` ]; then
    CORENUM=`lscpu | grep "Core(s) per socket" | awk -F: '{print $2}'`
  fi
  if [ -z ${CORENUM} ]; then
    cn=$(cat /sys/devices/system/node/node0/cpulist | sed 's|.*-||')
    ((cn+=1))
    CORENUM=${cn}
  fi

  if [ ! -z `command -v numactl` ]; then
    NUMANUM=`numactl -H | grep available | awk '{print $2}'`
  else
    NUMACTL_PRESENT=""
  fi
  if [ -z ${NUMANUM} ]; then
    nn=$(cat /sys/devices/system/node/online | sed 's|.*-||')
    ((nn+=1))
    NUMANUM=${nn}
  fi
else
  CORENUM=$(ls /sys/devices/system/cpu/ | grep "cpu[0-9]" | wc -l)
fi

#tt=$(expr ${NUMANUM} + 0 2>/dev/null) || echo NUMANUM=0
tt=$(expr ${NUMANUM} + 0 2>/dev/null)
[[ $NUMANUM -ge 2 ]] || NUMANUM=1
#dbg
#[[ ! -z $1 ]] && NUMANUM=$1
#[[ ! -z $2 ]] && CORENUM=$2
(( N = $NUMANUM - 1 ))

run_single_mem() {
    CNT=1
    for i in 0x1 0x2 0x4 0x8 0x10 0x20 0x40 0x80; do
	#echo "taskset $i ${MEMTEST} -s $MEMSIZE -c 0 -q &"
	taskset $i ${MEMTEST} &
	(( CNT = $CNT + 1 ))
	if [[ $CNT -gt $1 ]]; then
	    break
	fi
    done
}

run_numa_mem() {
    if [[ -z $NUMACTL_PRESENT ]]; then
	${MEMTEST} &
    else
	#echo "numactl --cpunodebind $1 --membind $2 ${MEMTEST} -s $MEMSIZE -c 0 -q &"
	numactl --cpunodebind $1 --membind $2 ${MEMTEST} &
    fi
}

killmem()
{
	killall -9 mem 2>/dev/null
}
#reset sic regs if sic value equal 0x0
init_sic_regs() {
    cnt=0
    echo "== Init SIC regs"
    for cur_ipcc_str in $IPCC_STR; do
	echo $cur_ipcc_str > $Path_SICw
	out[0]=`cat $Path_SICr | grep "node: 0 reg"`
	out[1]=`cat $Path_SICr | grep "node: 1 reg"`
	out[2]=`cat $Path_SICr | grep "node: 2 reg"`
	out[3]=`cat $Path_SICr | grep "node: 3 reg"`
	node=`cat $Path_SICr | wc -l`
	for cur_node in `seq 0 $((node - 1))`; do
	    reg_val=$(echo ${out[${cur_node}]} | awk '{print $5}')
	    if [[ $reg_val == "0x0" ]]; then
		echo "$cur_node $cur_ipcc_str 0xa0000000" > $Path_SICW
		continue
	    fi
	done
    done
}

FRES=
FSTR=
READ_ATTEMPT_NUMBER=20

get_reg() {
    local N=0
    local reg
    local reg_val
    local hex1=$(printf "0x%x" $1)
    local node=$2
    while true; do
	((N++))
	if [[ ! -z $3 ]] && [[ ! -z $ONE_MC_CH_REG ]]; then
		local hex_ch_num=$(printf "0x%x" $3)
		echo "$node 0x400 ${hex_ch_num}" > $Path_SICW
	fi
	echo ${hex1} > $Path_SICw
	FSTR=`cat ${Path_SICr} | grep "node: $node reg"`
	reg=(`echo ${FSTR,,} | awk '{print $4}' | tr '():' ' '`)
	reg_val=(`echo ${FSTR,,} | awk '{print $5}'`)
	if [[ $hex1 = ${reg[0]} ]]; then
	    break
	fi
	if [[ -z ${reg[0]} ]]; then
	    break
	fi
	if ((N>${READ_ATTEMPT_NUMBER})); then
	    break
	fi
#	echo "loop $N -- _${hex1}_ _${a[0]}_ __${a[*]}__"
	sleep .1
    done
    FRES=$(echo ${reg_val})
}

check_sic_regs() {
    ipl=(A B C)
    # turn off split string

    err=0
    local node=`cat $Path_SICr | wc -l`
    echo "== Check IPLINKS status"
    for i in ${!ipl[@]}; do
	echo -e "\tLink ${ipl[${i}]}"
	for cur_node in `seq 0 $((node - 1))`; do
	    get_reg ${IPCC_STR[${i}]} $cur_node
	    out[$cur_node]=$FRES
	    outstr[$cur_node]=$FSTR
	    get_reg ${IPCC_CSR[${i}]} $cur_node
	    out_csr[$cur_node]=$FRES
	    outstr_csr[$cur_node]=$FSTR
	done
	for cur_node in `seq 0 $((node - 1))`; do
	    if [ ! -z "${out_csr[${cur_node}]}" ]; then
		active=`printf "0x%x" $(( (${out_csr[${cur_node}]} & 0x80000000) >> 31 ))`
		width=`printf "0x%x" $((  (${out_csr[${cur_node}]} & 0x0F000000) >> 24 ))`
		state=`printf "0x%x" $((  (${out_csr[${cur_node}]} & 0x00070000) >> 16 ))`
		if ((!$active)); then
			echo -e "\t\t${outstr_csr[${cur_node}]} <-- link is not active"
			continue
		fi
		echo $state | grep -w -E ${CSR_CMD_CODE} >/dev/null
		if ((!$?)); then
			case "$state" in
			    "0x0" ) echo -e "\t\t${outstr_csr[${cur_node}]} <-- link state: PowerOff";;
			    "0x1" ) echo -e "\t\t${outstr_csr[${cur_node}]} <-- link state: Disable";;
			    "0x2" ) echo -e "\t\t${outstr_csr[${cur_node}]} <-- link state: Sleep";;
			esac
			continue
		fi
		if [ "$width" != '0xf' ]; then
			echo -e "\t\t${outstr_csr[${cur_node}]} <-- WARNING (not full width)"
		fi
	    else
		echo -e "\t\tWARNING (cannot find link ${ipl[${i}]} register with addr ${IPCC_STR[${i}]})}"
	    fi

	    if [[ ${out[${cur_node}]} != "0x40000000" ]] && [[ ${out[${cur_node}]} != "0x80000000" ]]; then
		(( i = ((${i} + 1)) * 10 + $cur_node ))
		elnk[${i}]="Link: ${ipl[${i}]} ${outstr[${cur_node}]} <-- link ERROR"
		echo -e "\t\t${outstr[${cur_node}]} <-- link ERROR"
		(( err = $err + 1 ))
	    else
		if [[ ${out[${cur_node}]} == "0x40000000" ]]; then
			echo -e "\t\t${outstr[${cur_node}]} <-- link WARNING"
		else
			echo -e "\t\t${outstr[${cur_node}]}"
		fi
	    fi
	done

    done

    return $err
}

check_sic_regs_module() {
  echo "== Check IPLINKS status"
  i=0
  err=0
  for cur_node in `seq 0 $((node - 1))`; do
	echo -e "\tNODE $cur_node"
	j=0
	for num_link in {A..C}; do
		case $num_link in
			"A")
				number_link=1;;
			"B")
				number_link=2;;
			"C")
				number_link=3;;
		esac
		width=`cat ${Path_to_link_info[$cur_node]} \
		       	| grep "NODE$cur_node-ipcc-$num_link-csr"  \
			| awk '{print $2}'`
		width=${width:6:3}
		active=`cat ${Path_to_link_info[$cur_node]} \
			| grep "NODE$cur_node-ipcc-$num_link-csr" \
			| awk '{print $7}' | tr -dc '0-9'`
		state=`cat ${Path_to_link_info[$cur_node]} \
			| grep "NODE$cur_node-ipcc-$num_link-csr" \
			| awk '{print $8}' | tr -dc '0-9'`
		bitrate=`cat ${Path_to_link_info[$cur_node]} \
			| grep "NODE$cur_node-ipcc-$num_link-csr" \
			| awk '{print $3}'`
		csr_reg=`cat ${Path_to_link_info[$cur_node]} \
			| grep "NODE$cur_node-ipcc-$num_link-csr" \
			| awk '{print $1}' | tr -dc '0-9' | cut -c 3-`
		str_reg=`cat ${Path_to_link_info[$cur_node]} \
			| grep "NODE$cur_node-ipcc-$num_link-str" \
			| awk '{print $1}'`
		str_reg=${str_reg:17:5}
		str_val=`cat ${Path_to_link_info[$cur_node]} \
			| grep "NODE$cur_node-ipcc-$num_link-str" \
			| awk '{print $4}' | tr -dc '0-9, a-f' | cut -c 2-`
		active_check[$j]=$active
		j=$((j+1))
		if [ $active -eq 0 ]; then
			print="Link: $num_link reg (0x$csr_reg): "
			print+="0x$str_val <-- link is not active"
			echo -e "\t\t$print"
		fi
		case "$state" in
			"0")
				print="Link: $num_link reg (0x$csr_reg): "
				print+="0x$str_val <-- link state: PowerOff"
				echo -e "\t\t$print";;
			"1")
				print="Link: $num_link reg (0x$csr_reg): "
				print+="0x$str_val <-- link state: Disable"
				echo -e "\t\t$print";;
			"2")
				print="Link: $num_link reg (0x$csr_reg): "
				print+="0x$str_val <-- link state: Sleep"
				echo -e "\t\t$print";;
		esac
		if [ "$width" != '0xf' ]; then
			print="Link: $num_link reg ($csr_reg): "
			print+="0x$str_val <-- WARNING (not full width)"
			echo -e "\t\t$print"
		fi
		if [ $active != 0 ]; then
			if [[ $str_val != "80000000" ]] && [[ $str_val != "40000000" ]]; then
				((i = ((${cur_node}+1))*10 + ${number_link} ))
				link_err_mess[${i}]="Node: $cur_node Link $num_link: "
				link_err_mess[${i}]+="reg (${str_reg}): 0x$str_val "
				link_err_mess[${i}]+="<-- link ERROR"
				print="Link: $num_link reg ($str_reg): "
				print+="0x$str_val <-- link ERROR"
				echo -e "\t\t$print"
				((err = $err + 1))
			else
				if [[ $str_val == "40000000" ]]; then
					print="Link: $num_link reg ($str_reg): "
					print+="0x$str_val  <-- link WARNING"
					echo -e "\t\t$print"
				else
					print="Link: $num_link reg "
					print+="($str_reg): 0x$str_val"
					echo -e "\t\t$print"
				fi
			fi
		fi
		if [ $DETAILED_FLAG -eq 1 ]; then
			echo -e "\t\tLink: $num_link rate: $bitrate Gbit/s"
		fi
	done

	if [ $node -eq 2 ]; then
		if [[ ${active_check[0]} -eq 0 ]] && [[ ${active_check[1]} -eq 0 ]] && \
						[[ ${active_check[2]} -eq 0 ]]; then
 			((i = ((${cur_node}+1))*100 + ${number_link} ))
			link_err_mess[${i}]="Node: $cur_node All links are not active "
			link_err_mess[${i}]+="<-- ERROR"
			echo -e "\t\tAll links are not active <-- ERROR"
			(( err = $err + 1 ))
		fi
	elif [ $node -eq 4 ]; then
		if [[ ${active_check[0]} -eq 0 ]] || [[ ${active_check[1]} -eq 0 ]] || \
						[[ ${active_check[2]} -eq 0 ]]; then
 			((i = ((${cur_node}+1))*100 + ${number_link} ))
			link_err_mess[${i}]="Node: $cur_node At least 1 link is not active "
			link_err_mess[${i}]+="<-- ERROR"
			echo -e "\t\tAt least 1 link is not active  <-- ERROR"
			(( err = $err + 1 ))
		fi
	fi
  done
  return $err
}

#get number of memory channels
check_ecc_regs() {
    err=0
    cnt=0
    local high_channel_num=$1
    local node=`cat $Path_SICr | wc -l`
    echo "== Check ECC status"
    for cur_ch_num in `seq 0 $high_channel_num`; do
	echo -e "\tChannel $cur_ch_num"
	if [[ ! -z $ONE_MC_CH_REG ]]; then
		mc_ecc_reg="0x440"
	else
		(( mc_ecc_reg = 0x400 + 0x40 * $cur_ch_num ))
	fi
	for cur_node in `seq 0 $((node - 1))`; do
	    get_reg $mc_ecc_reg $cur_node $cur_ch_num
	    out[$cur_node]=$FRES
	    outstr[$cur_node]=$FSTR
	done
	for cur_node in `seq 0 $((node - 1))`; do
	    #check MC_ECC[0} bit for ECC enable
	    ecc_on=$((1 & ${out[${cur_node}]}))
	    if [[ $ecc_on -eq 1 ]]; then
		    #check MC_ECC[16:31} bits for errors count
		    ecc_count=$((0xffff0000 & ${out[${cur_node}]}))
		    if [[ $ecc_count -ne 0 ]]; then
			    (( i = ((${cnt} + 1)) * 10 + $cur_node ))
			    ecnh[${i}]="Channel: ${cnt} ${outstr[${cur_node}]} <-- ECC ERROR"
			    echo -e "\t\t${outstr[${cur_node}]} <-- ECC ERROR"
			    (( err = $err + 1 ))
		    else
			    echo -e "\t\t${outstr[${cur_node}]}"
		    fi
	    else
		if [[ ${CHECK_MEMORY_PRESENT} -eq 1 ]]; then
			(( i = ((${cnt} + 1)) * 10 + $cur_node ))
			ecnh[${i}]="Channel: ${cnt} ${outstr[${cur_node}]} <-- MEMORY SLOT EMPTY"
			echo -e "\t\t${outstr[${cur_node}]} <-- MEMORY SLOT EMPTY"
			(( err = $err + 1 ))
		else
			echo -e "\t\t${outstr[${cur_node}]} <-- INCORRECT ECC MODE"
		fi
	    fi
	done

	(( cnt = $cnt + 1 ))
    done

    return $err
}

check_ecc_regs_module() {
 err=0
 i=0
 local node=`cat $Path_SICr | wc -l`
 echo "== Check ECC status"
 for cur_node in `seq 0 $((node - 1))`; do
	echo -e "\tNODE $cur_node"
	for cur_ecc in `seq 0 $((MEM_CHANNELS - 1))`; do
		ecc_on=`cat ${Path_to_mem_info[$cur_node]} \
			| grep "NODE-${cur_node}_MC${cur_ecc}" \
			| awk '{print $2}' | tr -dc '0-9'`
		ecc_count=`cat ${Path_to_mem_info[$cur_node]} \
			| grep "NODE-${cur_node}_MC${cur_ecc}" \
			| awk '{print $3}' | tr -dc '0-9'`
		ecc_count=${ecc_count:1:1}
		ecc_reg=`cat ${Path_to_mem_info[$cur_node]} \
			| grep "NODE-${cur_node}_MC${cur_ecc}" \
			| awk '{print $1}'`
		ecc_reg=${ecc_reg:15:5}
		if [ $DETAILED_FLAG -eq 1 ]; then
			mem_rate=`cat ${Path_to_mem_rate[$cur_node]} \
				| grep "MC_${cur_ecc}" | awk '{print $4}'`
			mem_freq=`cat ${Path_to_mem_rate[$cur_node]} \
				| grep "MC_${cur_ecc}" | awk '{print $6}'`
		fi
		if [[ $ecc_on -eq 1 ]]; then
			if [[ $ecc_count -ne 0 ]]; then
				((err = $err + 1))
				((i = ((${cur_node}+1))*10 + ${cur_ecc} ))
				mem_err_mess[${i}]="Node: $cur_node Channel: "
				mem_err_mess[${i}]+="$cur_ecc reg (${ecc_reg})"
				mem_err_mess[${i}]+=": 0x$ecc_on <-- ECC ERROR"
				print="Channel: $cur_ecc reg "
				print+="(${ecc_reg}): 0x$ecc_on <-- ECC ERROR"
				echo -e "\t\t$print"
			else
				print="Channel: $cur_ecc "
				print+="reg (${ecc_reg}): 0x$ecc_on"
				echo -e "\t\t$print"
			fi

		else
			if [[ ${CHECK_MEMORY_PRESENT} -eq 1 ]]; then
				((i = ((${cur_node}+1))*10 + ${cur_ecc} ))
				mem_err_mess[${i}]="Node: $cur_node Channel: "
				mem_err_mess[${i}]+="$cur_ecc reg (${ecc_reg})"
				mem_err_mess[${i}]+=": 0x$ecc_on <-- MEMORY "
				mem_err_mess[${i}]+="SLOT EMPTY"
				print="Channel: $cur_ecc reg (${ecc_reg})"
				print+=": 0x$ecc_on <-- MEMORY SLOT EMPTY"
				echo -e "\t\t$print"
				((err = $err + 1))
			else
				print="Channel: $cur_ecc reg (${ecc_reg})"
				print+=": 0x$ecc_on <-- INCORRECT ECC MODE"
				echo -e "\t\t$print"
			fi
		fi

		if [ $DETAILED_FLAG -eq 1 ]; then
			print="Channel: $cur_ecc rate: "
			print+="$mem_rate (frequency $mem_freq MHz)"
			echo -e "\t\t$print"
		fi
	done
 done

	return $err
}

#get numa numbers and core numbers
run_mem_tests() {
    (($NOMEMTEST)) && return
    echo "== Run memory tests"

    if [[ $1 -eq 1 ]]; then
	run_single_mem $2
    else
	for NODE in `seq 0 $N`; do
	    (( LOOP = $2 - 1 ))
	    while [[ $LOOP -ge 0 ]]; do
		for i in `seq 0 $N`; do
		    if [[ $LOOP -lt 0 ]]; then
			#echo "stop. no more cpu"
			break
		    fi
		    run_numa_mem $NODE $i
		    (( LOOP = $LOOP - 1 ))
		done
	    done
	done
    fi
}

print_date() {
    d=`date`
    echo "Current date: $d"
    [[ ! -z $1 ]] && echo "Finish  date: `date -d @$1`"
    echo
}
	echo -e "\tLink ${ipl[${cnt}]}"

print_results() {
goodprint=1
echo
echo "Summary results:"
if [  $active_flag -eq 0 ]; then
	if [[ $LINK_ERROR -gt $COUNT_ERROR ]]; then
		for i in `seq 0 3`; do
		    for j in `seq 0 3`; do
			(( l = ((${i} + 1)) * 10 + $j ))
			if [[ ! -z ${ecnh[${l}]} ]]; then
			    echo ${ecnh[${l}]}
			    goodprint=0
			fi
		    done
		done
		for i in `seq 0 3`; do
		    for j in `seq 0 3`; do
			(( l = ((${i} + 1)) * 10 + $j ))
			if [[ ! -z ${elnk[${l}]} ]]; then
			    echo ${elnk[${l}]}
			    goodprint=0
			fi
	   	    done
		done
	fi
else
	if [[ $LINK_ERROR -gt $COUNT_ERROR ]]; then
		for cur_node in `seq 0 $((node - 1))`; do
			for cur_ecc in `seq 0 $((MEM_CHANNELS - 1))`; do
				((l = (${cur_node}+1)*10 + ${cur_ecc} ))
				if [[ ! -z ${mem_err_mess[${l}]} ]]; then
					echo ${mem_err_mess[${l}]}
		    			goodprint=0
				fi
			done
		done
		for cur_node in `seq 0 $((node - 1))`; do
			for num_link in {A..C}; do
				case $num_link in
					"A")
						number_link=1;;
					"B")
						number_link=2;;
					"C")
						number_link=3;;
				esac

				((l = (${cur_node}+1)*10 + ${number_link} ))
				if [[ ! -z ${link_err_mess[${l}]} ]]; then
					echo ${link_err_mess[${l}]}
		    			goodprint=0
				fi

				((k = (${cur_node}+1)*100 + ${number_link} ))
				if [[ ! -z ${link_err_mess[${k}]} ]]; then
					echo ${link_err_mess[${k}]}
		    			goodprint=0
				fi

			done
		done
	fi
fi
	(($goodprint)) && echo -e "\terrors not found."

}

killmem
run_mem_tests $NUMANUM $CORENUM

if [  $active_flag -eq 0 ]; then
	init_sic_regs
fi

starttime=`date +%s`
(( endtime = $starttime + $WORKTIME ))
echo "======================= ======================="
print_date $endtime
while true; do
    if [[ $active_flag -eq 0 ]]; then
    	check_ecc_regs $((MEM_CHANNELS - 1))
    else
	check_ecc_regs_module
    fi

    if (($?)); then
	RET=1
	(($ONE_ERROR)) && break
    fi

    if [[ $active_flag -eq 0 ]]; then
    	check_sic_regs $((MEM_CHANNELS - 1))
    else
	check_sic_regs_module
    fi
    CURLINK_ERROR=$?
    (( LINK_ERROR = $LINK_ERROR + $CURLINK_ERROR ))
    if (($CURLINK_ERROR)); then
	RET=1
	(($ONE_ERROR)) && break
    fi
    curtime=`date +%s`
    if [[ $curtime -ge $endtime ]]; then
	break
    fi
    sleep $CHECKTIME
    echo "======================= ======================="
    print_date $endtime
done
echo "======================= ======================="

killmem
print_results
exit $RET
