#!/usr/bin/env python3

# Test keyfn ver.1.1
# Check all functional keys (max 65535 keys)
# defined in devtree gpio-keys-polled section

import struct
import array
import time
import sys
import os
import time


RET_PASS = 0
RET_FAIL = 1
RET_SKIP = 2

globalmask = 0
globalcodes = {} # {linux,code: [i, 'label']}


def print_f(msg):
  print(msg)
  sys.stdout.flush()


def findkeycodes():
  global globalmask
  global globalcodes
  maindir = "/sys/firmware/devicetree/base/gpio_keys_polled"

  with os.scandir(maindir) as files:
    subdir = [file.name for file in files if file.is_dir()]

  for i, item in enumerate(subdir):
    keylabel = ""
    keycode = 0
    with open(maindir + "/" + item + "/label") as file:
      keylabel = file.read()
    with open(maindir + "/" + item + "/linux,code", "rb") as file:
      code = struct.unpack('>i', file.read())
      keycode = code[0]
    globalcodes[keycode] = (i, keylabel)
    globalmask |= 1 << i

  globalmask = globalmask << 16 | globalmask

  print_f('Functional keys test, found key:')
  for key, val in globalcodes.items():
    print_f(f"  {val[0] + 1} code:{key} label:{val[1]}")


def checkkeys(in_file):
  global globalmask
  global globalcodes

  keymask = 0
  FORMAT = 'llHHI' # long, long, ushort, ushort, uint
  EVENT_SIZE = struct.calcsize(FORMAT)
  result = RET_FAIL

  try:
    event = in_file.read(EVENT_SIZE)
    while event:
      (tv_sec, tv_usec, type, code, value) = struct.unpack(FORMAT, event)

      val = globalcodes.get(code, None)
      if val != None:
        if   type == 1 and value == 1:
            print_f(f"{val[1]} down")
            keymask |= 1 << val[0]
        elif type == 1 and value == 0:
            print_f(f"{val[1]} up")
            keymask |= (1 << val[0]) << 16

      ##DEBUG
      #if type != 0 or code != 0 or value != 0:
      #    print_f("Event type %u, code %u, value %u at %d.%d" % \
      #        (type, code, value, tv_sec, tv_usec))
      #else:
      #    # Events with code, type and value == 0 are "separator" events
      #    print_f("===========================================")

      # all pressed
      if keymask == globalmask:
          result = RET_PASS
          break

      event = in_file.read(EVENT_SIZE)

  except (KeyboardInterrupt, Exception) as e:
    print_f('Terminated by user')
    pass

  return result


if __name__ == '__main__':
  infile_path = "/dev/input/by-path/platform-gpio_keys_polled-event"

  try:
    in_file = open(infile_path, "rb") # open in binary mode
  except FileNotFoundError as e:
    print(f"File {infile_path} not found!", file=sys.stderr)
    print_f('Result: SKIP')
    sys.exit(RET_SKIP)

  findkeycodes()

  print_f('Please, press all functional keys')
  result = checkkeys(in_file)

  in_file.close()

  if result == RET_PASS:
    print_f('All functional keys are checked')
    print_f('Result: SUCCESS')
  else:
    print_f('Result: FAIL')

  sys.exit(result)
