#!/bin/bash

umask 002

DIR=$(dirname `realpath $0`)


check_rtc_work() {
	echo "Check RTC time"
	tc=$(cat /sys/class/rtc/rtc0/since_epoch)
	for i in 0 1 2 3 4; do
		sleep 1
		tl=$(cat /sys/class/rtc/rtc0/since_epoch)
		printf "."
		if [[ $tl -lt $tc ]]; then
			echo
			echo "Error: RTC time not changed."
			return 1
		fi
		tc=$tl
	done
	echo
	return 0
}

check_os_time_change() {
	echo "Check OS time"
	hwt0=$(cat /sys/class/rtc/rtc0/date)
	curtime=$(date +%m%d%H%M%Y)
	cursec=$(date +%S)
	date 050711001981
	hwt1=$(cat /sys/class/rtc/rtc0/date)
	date ${curtime}.${cursec}
	if [[ $hwt0 -ne $hwt1 ]]; then
		echo "Error: change OS date corrupted RTC"
		return 1
	fi
	return 0
}

check_hwclock_time_change() {
	echo "Check change time"
	err=0
	hwclock -r
	r=$?
	(( err = $err + $r ))
	if (($r)); then
		echo "Error: read RTC from hwclock"
	fi
	hwt0=$(cat /sys/class/rtc/rtc0/date)
	curtime=$(date +%m%d%H%M%Y)
	cursec=$(date +%S)
	date 050711001981
	hwclock -w
	r=$?
	(( err = $err + $r ))
	if (($r)); then
		echo "Error: write hwclock to RTC"
	fi
	hwt1=$(cat /sys/class/rtc/rtc0/date)
	date ${curtime}.${cursec}
	hwclock -w
	r=$?
	(( err = $err + $r ))
	if [[ $hwt0 -eq $hwt1 ]] || (($r)); then
		(( err = $err + $r ))
		echo "Error: write hwclock to RTC"
	fi

	return $err
}

ERR=0

check_rtc_work
(( ERR = $? + $ERR ))

check_os_time_change
(( ERR = $? + $ERR ))

check_hwclock_time_change
(( ERR = $? + $ERR ))

if (($ERR)); then
    echo "Test: ERROR"
else
    echo "Test: SUCCESS"
fi

exit $ERR
