#!/bin/bash

# list for all devices
DEVICE="0000:01:0c.0"

RES=0
COUNT=0

device_detect() {
    DEVICES=$(lspci -D -d 1fff:800e 2>/dev/null | cut -d" " -f1 | head -1)
}

test_spmc() {

    name=$1
    device=$2

    echo "Test for SPMC ACPI PM $name"
    echo "For device: $device"

    acpidpid=`ps ax | grep "\/usr\/sbin\/acpid" | awk '{p=(\$1)} END {print p}'`

    if [[ ! -z $acpidpid ]]; then
    	echo "ACPI daemon is running with pid $acpidpid"
	echo "Stop acpid"
	kill -9 $acpidpid > /dev/null 2>&1
    fi
    fuser -k /proc/acpi/event

    echo "Init SCI:"

    echo "" > /var/log/spmc_${name}_${COUNT}.log
    sci_default_val=$(cat ${device}/sci)
    echo 1 > ${device}/sci

    echo "Do SCI interrupt:"
    if [[ $1 == "timer" ]]; then
	echo -e "\tTimer PM1_EN enabled. Please wait 10 seconds..."
	echo 0 > ${device}/tmr32
	echo 1 > ${device}/tmr
    else
	echo 1 > ${device}/pwrbtn
    fi

    cat /proc/acpi/event >> /var/log/spmc_${name}_${COUNT}.log &
    acpi_event_pid=$!
    echo
    if [[ $1 == "button" ]]; then
	echo -e "\tPlease - push POWER button !"
	echo -e "\tand then press ENTER key to start timer."
	read $r
    fi
    sleep 10
    kill -9 $acpi_event_pid >/dev/null 2>&1
    wait $acpi_event_pid 2>/dev/null

    echo "Check that SCI was handled"
    if [[ $1 == "timer" ]]; then
	st=$(grep -c "pmtimer" /var/log/spmc_${name}_${COUNT}.log)
    else
	st=$(grep -c "button" /var/log/spmc_${name}_${COUNT}.log)
    fi

    if [ $st -eq 0 ]; then
	echo ""
	echo "Failed to catch or handle SCI from $name."
	echo "Device: $device"
	echo "fail"
	echo ""
	loopres=1
	return
    else
	echo "SCI from $name caught and handled."
	echo "Device: $device"
	echo "pass"
	echo ""
	loopres=0
    fi

    if [[ $1 == "timer" ]]; then
	echo 0 > ${device}/tmr
    fi
    echo $sci_default_val > ${device}/sci
    if [[ ! -z $acpidpid ]]; then
	echo "Start acpid"
	/usr/sbin/acpid
    fi
}

usage()
{ IFS=''; while read helpstr
    do
	echo "$helpstr"
    done << EOF
Test SPMC.

Usage: $0 [-t] [-b]
 - no args  -- run test TIMER.
 -t         -- run test TIMER.
 -b         -- run test BUTTON.
 -h           -- help
EOF
}

ARG="timer"

OPTIONS="htb"
while getopts $OPTIONS OPT
do
    case $OPT in
	t ) ARG="timer";;
	b ) ARG="button";;
	h ) usage;exit 0;;
    esac
done

device_detect

for i in $DEVICES
do
    d=$(find /sys -name $i | head -1)
    loopres=1
    test_spmc $ARG $d
    if [[ $loopres -eq 1 ]]; then
	(( RES += 1 ))
    fi
    (( COUNT += 1 ))
done

echo
echo
if [[ $RES -eq 1 ]]; then
    echo "Test SPMC: FAIL"
else
    echo "Test SPMC: PASS"
fi

exit $RES
